{% extends "admin/change_form.html" %}

{% load i18n %}

{% block object-tools-items %}
{% if original %}<li><a href="{% url 'admin:form_designer_formsubmission_export' form_id=original.pk %}">{% trans "Export submissions" %}</a></li>{% endif %}
{{ block.super }}
{% endblock %}
